@@ -28,41 +28,41 @@
  *  FORWARD DECLARATIONS
  */
 
-static int Itk_PropagateOption _ANSI_ARGS_((Tcl_Interp *interp,
-    ItclObject *contextObj, ClientData cdata, CONST char *newval));
+static int Itk_PropagateOption (Tcl_Interp *interp,
+    ItclObject *contextObj, ClientData cdata, const char *newval);
 
-static int Itk_ArchSetOption _ANSI_ARGS_((Tcl_Interp *interp,
-    ArchInfo *info, CONST char *name, CONST char *value));
+static int Itk_ArchSetOption (Tcl_Interp *interp,
+    ArchInfo *info, const char *name, const char *value);
 
-static ArchComponent* Itk_CreateArchComponent _ANSI_ARGS_((
+static ArchComponent* Itk_CreateArchComponent (
     Tcl_Interp *interp, ArchInfo *info, char *name,
-    ItclClass *iclsPtr, Tcl_Command accessCmd));
-static void Itk_DelArchComponent _ANSI_ARGS_((ArchComponent *archComp));
+    ItclClass *iclsPtr, Tcl_Command accessCmd);
+static void Itk_DelArchComponent (ArchComponent *archComp);
 
-static int Itk_GetArchOption _ANSI_ARGS_((Tcl_Interp *interp,
+static int Itk_GetArchOption (Tcl_Interp *interp,
     ArchInfo *info, char *switchName, char *resName, char *resClass,
-    CONST char *defVal, char *currVal, ArchOption **aoPtr));
-static void Itk_InitArchOption _ANSI_ARGS_((Tcl_Interp *interp,
-    ArchInfo *info, ArchOption *archOpt, CONST char *defVal,
-    char *currVal));
-static void Itk_DelArchOption _ANSI_ARGS_((ArchOption *archOpt));
+    const char *defVal, char *currVal, ArchOption **aoPtr);
+static void Itk_InitArchOption (Tcl_Interp *interp,
+    ArchInfo *info, ArchOption *archOpt, const char *defVal,
+    char *currVal);
+static void Itk_DelArchOption (ArchOption *archOpt);
 
-static int Itk_RemoveArchOptionPart _ANSI_ARGS_((ArchInfo *info,
-    char *switchName, ClientData from));
-static int Itk_IgnoreArchOptionPart _ANSI_ARGS_((ArchInfo *info,
-    GenericConfigOpt *opt));
+static int Itk_RemoveArchOptionPart (ArchInfo *info,
+    char *switchName, ClientData from);
+static int Itk_IgnoreArchOptionPart (ArchInfo *info,
+    GenericConfigOpt *opt);
 
-static ConfigCmdline* Itk_CreateConfigCmdline _ANSI_ARGS_((
-    Tcl_Interp *interp, Tcl_Command accessCmd, char *switchName));
-static void Itk_DeleteConfigCmdline _ANSI_ARGS_((ClientData cdata));
+static ConfigCmdline* Itk_CreateConfigCmdline (
+    Tcl_Interp *interp, Tcl_Command accessCmd, char *switchName);
+static void Itk_DeleteConfigCmdline (ClientData cdata);
 
-static Tcl_HashTable* Itk_CreateGenericOptTable _ANSI_ARGS_((Tcl_Interp *interp,
-    const char *options));
-static void Itk_DelGenericOptTable _ANSI_ARGS_((Tcl_HashTable *tPtr));
+static Tcl_HashTable* Itk_CreateGenericOptTable (Tcl_Interp *interp,
+    const char *options);
+static void Itk_DelGenericOptTable (Tcl_HashTable *tPtr);
 
-static GenericConfigOpt* Itk_CreateGenericOpt _ANSI_ARGS_((Tcl_Interp *interp,
-    const char *switchName, Tcl_Command accessCmd));
-static void Itk_DelGenericOpt _ANSI_ARGS_((GenericConfigOpt* opt));
+static GenericConfigOpt* Itk_CreateGenericOpt (Tcl_Interp *interp,
+    const char *switchName, Tcl_Command accessCmd);
+static void Itk_DelGenericOpt (GenericConfigOpt* opt);
 
 
 
@@ -78,7 +78,7 @@ static void Itk_DelGenericOpt _ANSI_ARGS_((GenericConfigOpt* opt));
  */
 void
 Itk_DelMergeInfo(
-    char* cdata)  /* data to be destroyed */
+    void* cdata)  /* data to be destroyed */
 {
     ArchMergeInfo *mergeInfo = (ArchMergeInfo*)cdata;
 
@@ -168,7 +168,7 @@ Itk_ArchCompAddCmd(
     ClientData dummy,        /* unused */
     Tcl_Interp *interp,      /* current interpreter */
     int objc,                /* number of arguments */
-    Tcl_Obj *CONST objv[])   /* argument objects */
+    Tcl_Obj *const objv[])   /* argument objects */
 {
     Tcl_HashEntry *entry = NULL;
     char *path = NULL;
@@ -180,13 +180,11 @@ Itk_ArchCompAddCmd(
     Tcl_Obj *hullNamePtr = NULL;
     int pLevel = ITCL_PUBLIC;
 
-    ItclShowArgs(1, "Itk_ArchCompAddCmd", objc, objv);
     int newEntry;
     int result;
-    CONST char *cmd;
-    CONST char *token;
-    CONST char *resultStr;
-    Tcl_CallFrame frame;
+    const char *cmd;
+    const char *token;
+    const char *resultStr;
     char *name;
     Tcl_Namespace *parserNs;
     ItclClass *contextClass;
@@ -196,11 +194,8 @@ Itk_ArchCompAddCmd(
     Tcl_Command accessCmd;
     Tcl_Obj *objPtr;
     Tcl_DString buffer;
-    Tcl_CallFrame *uplevelFramePtr;
-    Tcl_CallFrame *oldFramePtr = NULL;
-    ItclObjectInfo *infoPtr;
-    ItclCallContext *callContextPtr;
 
+    ItclShowArgs(1, "Itk_ArchCompAddCmd", objc, objv);
     /*
      *  Get the Archetype info associated with this widget.
      */
@@ -292,7 +287,7 @@ Itk_ArchCompAddCmd(
             contextObj->accessCmd, tmpNamePtr);
         Tcl_AppendToObj(tmpNamePtr, "-widget-", -1);
         Tcl_IncrRefCount(tmpNamePtr);
-        
+
         result = Itcl_RenameCommand(interp, Tcl_GetString(objNamePtr),
                 Tcl_GetString(tmpNamePtr));
 
@@ -305,10 +300,6 @@ Itk_ArchCompAddCmd(
      *  Execute the <createCmds> to create the component widget.
      *  Do this one level up, in the scope of the calling routine.
      */
-    Itcl_SetCallFrameResolver(interp, contextClass->resolvePtr);
-    infoPtr = Tcl_GetAssocData(interp, ITCL_INTERP_DATA, NULL);
-    uplevelFramePtr = Itcl_GetUplevelCallFrame(interp, 1);
-    oldFramePtr = Itcl_ActivateCallFrame(interp, uplevelFramePtr);
     result = Tcl_EvalObjEx(interp, objv[2], 0);
     if (result != TCL_OK) {
         goto compFail;
@@ -320,7 +311,7 @@ Itk_ArchCompAddCmd(
      *  this, since the interpreter will get used in the mean time.
      */
     resultStr = Tcl_GetStringResult(interp);
-    path = (char*)ckalloc((unsigned)(strlen(resultStr)+1));
+    path = (char*)ckalloc((strlen(resultStr)+1));
     strcpy(path, resultStr);
 
     /*
@@ -339,8 +330,6 @@ Itk_ArchCompAddCmd(
         goto compFail;
     }
 
-    (void) Itcl_ActivateCallFrame(interp, oldFramePtr);
-    oldFramePtr = NULL;
     winNamePtr = Tcl_NewStringObj((char*)NULL, 0);
     Tcl_GetCommandFullName(interp, accessCmd, winNamePtr);
     Tcl_IncrRefCount(winNamePtr);
@@ -351,21 +340,6 @@ Itk_ArchCompAddCmd(
      *  according to the "-protected" or "-private" option.
      */
     ownerClass = contextClass;
-    Tcl_Namespace *ownerNsPtr;
-    callContextPtr = Itcl_PeekStack(&infoPtr->contextStack);
-    ownerNsPtr = callContextPtr->nsPtr;
-    if (ownerNsPtr != NULL) {
-        Tcl_HashEntry *hPtr;
-	int idx = 2;
-	if (Itcl_GetStackSize(&infoPtr->contextStack) == 1) {
-	   idx = 1;
-	}
-        callContextPtr = Itcl_GetStackValue(&infoPtr->contextStack,
-	        Itcl_GetStackSize(&infoPtr->contextStack)-idx);
-        hPtr = Tcl_FindHashEntry(&infoPtr->namespaceClasses,
-                (char *)callContextPtr->nsPtr);
-        ownerClass = (ItclClass*)Tcl_GetHashValue(hPtr);
-    }
 
     archComp = Itk_CreateArchComponent(interp, info, name, ownerClass,
             accessCmd);
@@ -417,16 +391,21 @@ Itk_ArchCompAddCmd(
          *  these things are not really necessary.
          */
         Tcl_DStringSetLength(&buffer, 0);
-        Tcl_DStringAppend(&buffer, "bindtags ", -1);
+        Tcl_DStringAppend(&buffer, "::bindtags ", -1);
         Tcl_DStringAppend(&buffer, path, -1);
         if (Tcl_Eval(interp, Tcl_DStringValue(&buffer)) != TCL_OK) {
             goto compFail;
         }
 
+	/*
+	 * NOTE: We need the [::itcl::code] because the itk_component
+	 * method is protected.
+	 */
+
         Tcl_DStringSetLength(&buffer, 0);
-        Tcl_DStringAppend(&buffer, "bind itk-destroy-", -1);
+        Tcl_DStringAppend(&buffer, "::bind itk-destroy-", -1);
         Tcl_DStringAppend(&buffer, path, -1);
-        Tcl_DStringAppend(&buffer, " <Destroy> [itcl::code ", -1);
+        Tcl_DStringAppend(&buffer, " <Destroy> [::itcl::code ", -1);
 
         Tcl_DStringAppend(&buffer,
             Tcl_GetStringFromObj(objNamePtr,(int*)NULL), -1);
@@ -434,13 +413,14 @@ Itk_ArchCompAddCmd(
         Tcl_DStringAppend(&buffer, " itk_component delete ", -1);
         Tcl_DStringAppend(&buffer, name, -1);
         Tcl_DStringAppend(&buffer, "]\n", -1);
-        Tcl_DStringAppend(&buffer, "bindtags ", -1);
+        Tcl_DStringAppend(&buffer, "::bindtags ", -1);
         Tcl_DStringAppend(&buffer, path, -1);
         Tcl_DStringAppend(&buffer, " {itk-destroy-", -1);
         Tcl_DStringAppend(&buffer, path, -1);
         Tcl_DStringAppend(&buffer, " ", -1);
         Tcl_DStringAppend(&buffer, Tcl_GetStringResult(interp), -1);
         Tcl_DStringAppend(&buffer, "}", -1);
+
         if (Tcl_Eval(interp, Tcl_DStringValue(&buffer)) != TCL_OK) {
             goto compFail;
         }
@@ -452,7 +432,7 @@ Itk_ArchCompAddCmd(
      *  table to represent these, so they can be found quickly
      *  by the option parsing commands in "itk::option-parser".
      */
-    Tcl_DStringTrunc(&buffer, 0);
+    Tcl_DStringSetLength(&buffer, 0);
     Tcl_DStringAppendElement(&buffer,
         Tcl_GetStringFromObj(winNamePtr, (int*)NULL));
     Tcl_DStringAppendElement(&buffer, "configure");
@@ -504,15 +484,11 @@ Itk_ArchCompAddCmd(
         objPtr = objv[3];
     }
 
-    result = Itcl_PushCallFrame(interp, &frame, parserNs,
-            /* isProcCallFrame */ 0);
+    Tcl_Eval(interp, "::namespace path [::lreplace [::namespace path] end+1 end ::itk::option-parser]");
+    result = Tcl_EvalObj(interp, objPtr);
+    Tcl_Eval(interp, "::namespace path [::lrange [::namespace path] 0 end-1]");
 
-    if (result == TCL_OK) {
-        result = Tcl_EvalObj(interp, objPtr);
-        Itcl_PopCallFrame(interp);
-    }
-
-    if (objPtr != objv[3]) {
+    if (objc != 4) {
         Tcl_DecrRefCount(objPtr);
     }
     if (result != TCL_OK) {
@@ -547,9 +523,6 @@ Itk_ArchCompAddCmd(
      *  If any errors were encountered, clean up and return.
      */
 compFail:
-    if (oldFramePtr) {
-	(void) Itcl_ActivateCallFrame(interp, oldFramePtr);
-    }
     if (archComp) {
         Itk_DelArchComponent(archComp);
     }
@@ -615,11 +588,11 @@ compFail:
  */
 /* ARGSUSED */
 int
-Itk_ArchCompDeleteCmd(dummy, interp, objc, objv)
-    ClientData dummy;        /* unused */
-    Tcl_Interp *interp;      /* current interpreter */
-    int objc;                /* number of arguments */
-    Tcl_Obj *CONST objv[];   /* argument objects */
+Itk_ArchCompDeleteCmd(
+    ClientData dummy,        /* unused */
+    Tcl_Interp *interp,      /* current interpreter */
+    int objc,                /* number of arguments */
+    Tcl_Obj *const objv[])   /* argument objects */
 {
     int i;
     char *token;
@@ -677,7 +650,7 @@ if (archComp == NULL) {
         *  Ignore errors if anything goes wrong.
         */
         Tcl_DStringInit(&buffer);
-        Tcl_DStringAppend(&buffer, "itk::remove_destroy_hook ", -1);
+        Tcl_DStringAppend(&buffer, "::itk::remove_destroy_hook ", -1);
         Tcl_DStringAppend(&buffer, archComp->pathName, -1);
         (void) Tcl_Eval(interp, Tcl_DStringValue(&buffer));
         Tcl_ResetResult(interp);
@@ -716,7 +689,7 @@ if (archComp == NULL) {
         elem = Itcl_FirstListElem(&delOptList);
         while (elem) {
             entry = (Tcl_HashEntry*)Itcl_GetListValue(elem);
-            token = Tcl_GetHashKey(&info->options, entry);
+            token = (char *)Tcl_GetHashKey(&info->options, entry);
 
             Itk_RemoveArchOptionPart(info, token, (ClientData)archComp);
 
@@ -748,11 +721,11 @@ if (archComp == NULL) {
  */
 /* ARGSUSED */
 int
-Itk_ArchOptKeepCmd(clientData, interp, objc, objv)
-    ClientData clientData;   /* option merging info record */
-    Tcl_Interp *interp;      /* current interpreter */
-    int objc;                /* number of arguments */
-    Tcl_Obj *CONST objv[];   /* argument objects */
+Itk_ArchOptKeepCmd(
+    ClientData clientData,   /* option merging info record */
+    Tcl_Interp *interp,      /* current interpreter */
+    int objc,                /* number of arguments */
+    Tcl_Obj *const objv[])   /* argument objects */
 {
     ArchMergeInfo *mergeInfo = (ArchMergeInfo*)clientData;
     int result = TCL_OK;
@@ -854,11 +827,11 @@ Itk_ArchOptKeepCmd(clientData, interp, objc, objv)
  */
 /* ARGSUSED */
 int
-Itk_ArchOptIgnoreCmd(clientData, interp, objc, objv)
-    ClientData clientData;   /* option merging info record */
-    Tcl_Interp *interp;      /* current interpreter */
-    int objc;                /* number of arguments */
-    Tcl_Obj *CONST objv[];   /* argument objects */
+Itk_ArchOptIgnoreCmd(
+    ClientData clientData,   /* option merging info record */
+    Tcl_Interp *interp,      /* current interpreter */
+    int objc,                /* number of arguments */
+    Tcl_Obj *const objv[])   /* argument objects */
 {
     ArchMergeInfo *mergeInfo = (ArchMergeInfo*)clientData;
 
@@ -929,11 +902,11 @@ Itk_ArchOptIgnoreCmd(clientData, interp, objc, objv)
  */
 /* ARGSUSED */
 int
-Itk_ArchOptRenameCmd(clientData, interp, objc, objv)
-    ClientData clientData;   /* option merging info record */
-    Tcl_Interp *interp;      /* current interpreter */
-    int objc;                /* number of arguments */
-    Tcl_Obj *CONST objv[];   /* argument objects */
+Itk_ArchOptRenameCmd(
+    ClientData clientData,   /* option merging info record */
+    Tcl_Interp *interp,      /* current interpreter */
+    int objc,                /* number of arguments */
+    Tcl_Obj *const objv[])   /* argument objects */
 {
     ArchMergeInfo *mergeInfo = (ArchMergeInfo*)clientData;
 
@@ -1059,15 +1032,15 @@ Itk_ArchOptRenameCmd(clientData, interp, objc, objv)
  */
 /* ARGSUSED */
 int
-Itk_ArchOptUsualCmd(clientData, interp, objc, objv)
-    ClientData clientData;   /* option merging info record */
-    Tcl_Interp *interp;      /* current interpreter */
-    int objc;                /* number of arguments */
-    Tcl_Obj *CONST objv[];   /* argument objects */
+Itk_ArchOptUsualCmd(
+    ClientData clientData,   /* option merging info record */
+    Tcl_Interp *interp,      /* current interpreter */
+    int objc,                /* number of arguments */
+    Tcl_Obj *const objv[])   /* argument objects */
 {
     ArchMergeInfo *mergeInfo = (ArchMergeInfo*)clientData;
 
-    CONST char *tag;
+    const char *tag;
     Tcl_HashEntry *entry;
     Tcl_Obj *codePtr;
 
@@ -1143,11 +1116,11 @@ Itk_ArchOptUsualCmd(clientData, interp, objc, objv)
  */
 /* ARGSUSED */
 int
-Itk_UsualCmd(clientData, interp, objc, objv)
-    ClientData clientData;   /* option merging info record */
-    Tcl_Interp *interp;      /* current interpreter */
-    int objc;                /* number of arguments */
-    Tcl_Obj *CONST objv[];   /* argument objects */
+Itk_UsualCmd(
+    ClientData clientData,   /* option merging info record */
+    Tcl_Interp *interp,      /* current interpreter */
+    int objc,                /* number of arguments */
+    Tcl_Obj *const objv[])   /* argument objects */
 {
     ArchMergeInfo *mergeInfo = (ArchMergeInfo*)clientData;
 
@@ -1170,7 +1143,7 @@ Itk_UsualCmd(clientData, interp, objc, objv)
     if (objc == 1) {
         entry = Tcl_FirstHashEntry(&mergeInfo->usualCode, &place);
         while (entry) {
-            tag = Tcl_GetHashKey(&mergeInfo->usualCode, entry);
+            tag = (char *)Tcl_GetHashKey(&mergeInfo->usualCode, entry);
             Tcl_AppendElement(interp, tag);
             entry = Tcl_NextHashEntry(&place);
         }
@@ -1189,11 +1162,11 @@ Itk_UsualCmd(clientData, interp, objc, objv)
                 codePtr = (Tcl_Obj*)Tcl_GetHashValue(entry);
                 Tcl_DecrRefCount(codePtr);
             }
-    
+
             codePtr = objv[2];
             Tcl_IncrRefCount(codePtr);
             Tcl_SetHashValue(entry, (ClientData)codePtr);
-    
+
             return TCL_OK;
         }
     }
@@ -1236,7 +1209,7 @@ Itk_ArchOptionAddCmd(
     ClientData dummy,        /* unused */
     Tcl_Interp *interp,      /* current interpreter */
     int objc,                /* number of arguments */
-    Tcl_Obj *CONST objv[])   /* argument objects */
+    Tcl_Obj *const objv[])   /* argument objects */
 {
     ItclClass *contextClass;
     ItclClass *iclsPtr;
@@ -1421,11 +1394,11 @@ Itk_ArchOptionAddCmd(
  */
 /* ARGSUSED */
 int
-Itk_ArchOptionRemoveCmd(dummy, interp, objc, objv)
-    ClientData dummy;        /* unused */
-    Tcl_Interp *interp;      /* current interpreter */
-    int objc;                /* number of arguments */
-    Tcl_Obj *CONST objv[];   /* argument objects */
+Itk_ArchOptionRemoveCmd(
+    ClientData dummy,        /* unused */
+    Tcl_Interp *interp,      /* current interpreter */
+    int objc,                /* number of arguments */
+    Tcl_Obj *const objv[])   /* argument objects */
 {
     ItclClass *contextClass;
     ItclClass *iclsPtr;
@@ -1570,7 +1543,7 @@ Itk_PropagateOption(
     Tcl_Interp *interp,        /* interpreter managing the class */
     ItclObject *contextObj,    /* itcl object being configured */
     ClientData cdata,          /* command prefix to use for configuration */
-    CONST char *newval)        /* new value for this option */
+    const char *newval)        /* new value for this option */
 {
     ConfigCmdline *cmdlinePtr = (ConfigCmdline*)cdata;
     int result;
@@ -1606,13 +1579,12 @@ Itk_PropagatePublicVar(
     Tcl_Interp *interp,        /* interpreter managing the class */
     ItclObject *contextObj,    /* itcl object being configured */
     ClientData cdata,          /* command prefix to use for configuration */
-    CONST char *newval)        /* new value for this option */
+    const char *newval)        /* new value for this option */
 {
     ItclVariable *ivPtr = (ItclVariable*)cdata;
 
-    Tcl_CallFrame frame;
-    int result;
-    CONST char *val;
+    int result = TCL_OK;
+    const char *val;
     ItclMemberCode *mcode;
 
     /*
@@ -1622,21 +1594,24 @@ Itk_PropagatePublicVar(
      *  is the most-specific class, so that the public variable can
      *  be found.
      */
-    result = Itcl_PushCallFrame(interp, &frame, contextObj->iclsPtr->nsPtr,
-            /*isProcCallFrame*/0);
 
     if (result == TCL_OK) {
 	/*
-	 * Casting away CONST of newval only to satisfy Tcl 8.3 and
+	 * Casting away const of newval only to satisfy Tcl 8.3 and
 	 * earlier headers.
 	 */
+
+#if 1
+	val = ItclSetInstanceVar(interp, Tcl_GetString(ivPtr->fullNamePtr),
+		NULL, newval, contextObj, ivPtr->iclsPtr);
+#else
         val = Tcl_SetVar2(interp, Tcl_GetString(ivPtr->fullNamePtr), (char *) NULL,
             (char *) newval, TCL_LEAVE_ERR_MSG);
+#endif
 
         if (!val) {
             result = TCL_ERROR;
         }
-        Itcl_PopCallFrame(interp);
     }
 
     if (result != TCL_OK) {
@@ -1654,12 +1629,15 @@ Itk_PropagatePublicVar(
      */
     mcode = ivPtr->codePtr;
     if (mcode && mcode->bodyPtr) {
+	Tcl_CallFrame frame;
+
+	Itcl_PushCallFrame(interp, &frame, ivPtr->iclsPtr->nsPtr, 1);
+	Itcl_SetContext(interp, contextObj);
 
-        Itcl_SetCallFrameResolver(interp, ivPtr->iclsPtr->resolvePtr);
-        Tcl_Namespace *saveNsPtr = Tcl_GetCurrentNamespace(interp);
-        Itcl_SetCallFrameNamespace(interp, ivPtr->iclsPtr->nsPtr);
         result = Tcl_EvalObjEx(interp, mcode->bodyPtr, 0);
-        Itcl_SetCallFrameNamespace(interp, saveNsPtr);
+
+	Itcl_UnsetContext(interp);
+	Itcl_PopCallFrame(interp);
 
         if (result == TCL_OK) {
             Tcl_ResetResult(interp);
@@ -1700,8 +1678,8 @@ static int
 Itk_ArchSetOption(
     Tcl_Interp *interp,        /* interpreter managing this widget */
     ArchInfo *info,            /* Archetype info */
-    CONST char *name,          /* name of configuration option */
-    CONST char *value)         /* new value for configuration option */
+    const char *name,          /* name of configuration option */
+    const char *value)         /* new value for configuration option */
 {
     Tcl_HashEntry *entry;
     ArchOption *archOpt;
@@ -1752,13 +1730,15 @@ Itk_ArchConfigOption(
     char *value)               /* new value for configuration option */
 {
     int result;
-    CONST char *v; 
+    const char *v;
     char *lastval;
     Tcl_HashEntry *entry;
     ArchOption *archOpt;
     Itcl_ListElem *part;
     ArchOptionPart *optPart;
     Itcl_InterpState istate;
+    ItclClass *iclsPtr;
+    ItclObject *ioPtr;
 
     /*
      *  Query the "itk_option" array to get the current setting.
@@ -1777,9 +1757,17 @@ Itk_ArchConfigOption(
     }
     archOpt = (ArchOption*)Tcl_GetHashValue(entry);
 
+#if 0
     v = Tcl_GetVar2(interp, "itk_option", archOpt->switchName, 0);
+#else
+    Itcl_GetContext(interp, &iclsPtr, &ioPtr);
+
+    v = ItclGetInstanceVar(interp, "itk_option", archOpt->switchName,
+	    ioPtr, iclsPtr);
+#endif
+
     if (v) {
-        lastval = (char*)ckalloc((unsigned)(strlen(v)+1));
+        lastval = (char*)ckalloc((strlen(v)+1));
         strcpy(lastval, v);
     } else {
         lastval = NULL;
@@ -1788,7 +1776,12 @@ Itk_ArchConfigOption(
     /*
      *  Update the "itk_option" array with the new setting.
      */
+#if 0
     if (!Tcl_SetVar2(interp, "itk_option", archOpt->switchName, value, 0)) {
+#else
+    if (!ItclSetInstanceVar(interp, "itk_option", archOpt->switchName, value,
+	    ioPtr, iclsPtr)) {
+#endif
         Itk_ArchOptAccessError(interp, info, archOpt);
         result = TCL_ERROR;
         goto configDone;
@@ -1820,7 +1813,12 @@ Itk_ArchConfigOption(
     if (result == TCL_ERROR) {
         istate = Itcl_SaveInterpState(interp, result);
 
+#if 0
         Tcl_SetVar2(interp, "itk_option", archOpt->switchName, lastval, 0);
+#else
+	ItclSetInstanceVar(interp, "itk_option", archOpt->switchName, lastval,
+	    ioPtr, iclsPtr);
+#endif
 
         part = Itcl_FirstListElem(&archOpt->parts);
         while (part) {
@@ -1864,8 +1862,8 @@ Itk_CreateArchComponent(
     ItclClass *iclsPtr,            /* component created in this class */
     Tcl_Command accessCmd)         /* access command for component */
 {
-    CONST char *init;
-    CONST char *wname;
+    const char *init;
+    const char *wname;
     ArchComponent *archComp;
     ArchOption *archOpt;
     Tk_Window tkwin;
@@ -1914,7 +1912,7 @@ Itk_CreateArchComponent(
                 if (!archOpt->init) {
                     ckfree(archOpt->init);
                 }
-                archOpt->init = (char*)ckalloc((unsigned)(strlen(init)+1));
+                archOpt->init = (char*)ckalloc((strlen(init)+1));
                 strcpy(archOpt->init, init);
 
                 if (Itk_ArchSetOption(interp, info,
@@ -1938,7 +1936,7 @@ Itk_CreateArchComponent(
     archComp->protection  = ITCL_PUBLIC;
     archComp->accessCmd  = accessCmd;
     archComp->tkwin      = tkwin;
-    archComp->pathName   = (char *) ckalloc((unsigned)(strlen(wname)+1));
+    archComp->pathName   = (char *) ckalloc((strlen(wname)+1));
     strcpy(archComp->pathName, wname);
 
     return archComp;
@@ -1992,7 +1990,7 @@ Itk_GetArchOption(
     char *switchName,              /* name of command-line switch */
     char *resName,                 /* resource name in X11 database */
     char *resClass,                /* resource class name in X11 database */
-    CONST char *defVal,            /* last-resort default value */
+    const char *defVal,            /* last-resort default value */
     char *currVal,                 /* current option value */
     ArchOption **aoPtr)            /* returns: option record */
 {
@@ -2007,7 +2005,7 @@ Itk_GetArchOption(
      *  If the switch does not have a leading "-", add it on.
      */
     if (*switchName != '-') {
-        name = ckalloc((unsigned)(strlen(switchName)+2));
+        name = (char *)ckalloc((strlen(switchName)+2));
         *name = '-';
         strcpy(name+1, switchName);
     } else {
@@ -2024,7 +2022,7 @@ Itk_GetArchOption(
         archOpt = (ArchOption*)Tcl_GetHashValue(entry);
 
         if (resName && !archOpt->resName) {
-            archOpt->resName = (char*)ckalloc((unsigned)(strlen(resName)+1));
+            archOpt->resName = (char*)ckalloc((strlen(resName)+1));
             strcpy(archOpt->resName, resName);
         }
         else if (resName && strcmp(archOpt->resName, resName) != 0) {
@@ -2037,7 +2035,7 @@ Itk_GetArchOption(
         }
 
         if (resClass && !archOpt->resClass) {
-            archOpt->resClass = (char*)ckalloc((unsigned)(strlen(resClass)+1));
+            archOpt->resClass = (char*)ckalloc((strlen(resClass)+1));
             strcpy(archOpt->resClass, resClass);
         } else {
 	    if (resClass && strcmp(archOpt->resClass, resClass) != 0) {
@@ -2065,18 +2063,18 @@ Itk_GetArchOption(
      */
     archOpt = (ArchOption*)ckalloc(sizeof(ArchOption));
 
-    archOpt->switchName = (char*)ckalloc((unsigned)(strlen(name)+1));
+    archOpt->switchName = (char*)ckalloc((strlen(name)+1));
     strcpy(archOpt->switchName, name);
 
     if (resName) {
-        archOpt->resName = (char*)ckalloc((unsigned)(strlen(resName)+1));
+        archOpt->resName = (char*)ckalloc((strlen(resName)+1));
         strcpy(archOpt->resName, resName);
     } else {
         archOpt->resName = NULL;
     }
 
     if (resClass) {
-        archOpt->resClass = (char*)ckalloc((unsigned)(strlen(resClass)+1));
+        archOpt->resClass = (char*)ckalloc((strlen(resClass)+1));
         strcpy(archOpt->resClass, resClass);
     } else {
         archOpt->resClass = NULL;
@@ -2122,14 +2120,11 @@ Itk_InitArchOption(
     Tcl_Interp *interp,            /* interpreter managing the object */
     ArchInfo *info,                /* info for Archetype mega-widget */
     ArchOption *archOpt,           /* option to initialize */
-    CONST char *defVal,            /* last-resort default value */
+    const char *defVal,            /* last-resort default value */
     char *currVal)                 /* current option value */
 {
-    CONST char *init = NULL;
-
-    Tcl_CallFrame frame;
-    int result;
-    CONST char *ival;
+    const char *init = NULL;
+    const char *ival;
     char c;
 
     /*
@@ -2169,25 +2164,11 @@ Itk_InitArchOption(
         ival = init;
     }
 
-    /*
-     *  Set the initial value in the itk_option array.
-     *  Since this might be called from the itk::option-parser
-     *  namespace, reinstall the object context.
-     */
-    result = Itcl_PushCallFrame(interp, &frame, info->itclObj->iclsPtr->nsPtr, /*isProcCallFrame*/0);
-
-    if (result == TCL_OK) {
-	/*
-	 * Casting away CONST of ival only to satisfy Tcl 8.3 and
-	 * earlier headers.
-	 */
-        Tcl_SetVar2(interp, "itk_option", archOpt->switchName,
+    Tcl_SetVar2(interp, "itk_option", archOpt->switchName,
             (char *)((ival) ? ival : ""), 0);
-    Itcl_PopCallFrame(interp);
-    }
 
     if (ival) {
-        archOpt->init = (char*)ckalloc((unsigned)(strlen(ival)+1));
+        archOpt->init = (char*)ckalloc((strlen(ival)+1));
         strcpy(archOpt->init, ival);
     }
 }
@@ -2298,16 +2279,15 @@ Itk_AddOptionPart(
     char *switchName,                /* name of command-line switch */
     char *resName,                   /* resource name in X11 database */
     char *resClass,                  /* resource class name in X11 database */
-    CONST char *defVal,              /* last-resort default value */
+    const char *defVal,              /* last-resort default value */
     char *currVal,                   /* current value (or NULL) */
     ArchOptionPart *optPart,         /* part to be added in */
     ArchOption **raOpt)              /* returns: option containing new part */
 {
-    CONST char *init = NULL;
-
-    Tcl_CallFrame frame;
+    const char *init = NULL;
     int result;
     ArchOption *archOpt;
+    Itcl_ListElem *elemPtr;
 
     *raOpt = NULL;
     archOpt = NULL;
@@ -2328,19 +2308,17 @@ Itk_AddOptionPart(
      *  simply update this part to the current value.  Otherwise,
      *  leave the configuration to Itk_ArchInitCmd().
      */
-    Itcl_AppendList(&archOpt->parts, (ClientData)optPart);
+    elemPtr = Itcl_AppendList(&archOpt->parts, (ClientData)optPart);
 
     if ((archOpt->flags & ITK_ARCHOPT_INIT) != 0) {
 
-        result = Itcl_PushCallFrame(interp, &frame, info->itclObj->iclsPtr->nsPtr, /*isProcCallFrame*/0);
-
         if (result == TCL_OK) {
             init = Tcl_GetVar2(interp, "itk_option", archOpt->switchName, 0);
-            Itcl_PopCallFrame(interp);
         }
 
         if (!init) {
             Itk_ArchOptAccessError(interp, info, archOpt);
+	    Itcl_DeleteListElem(elemPtr);
             return TCL_ERROR;
         }
 
@@ -2350,6 +2328,7 @@ Itk_AddOptionPart(
 
             if (result != TCL_OK) {
                 Itk_ArchOptConfigError(interp, info, archOpt);
+		Itcl_DeleteListElem(elemPtr);
                 return TCL_ERROR;
             }
         }
@@ -2391,7 +2370,7 @@ Itk_FindArchOptionPart(
      *  If the switch does not have a leading "-", add it on.
      */
     if (*switchName != '-') {
-        name = ckalloc((unsigned)(strlen(switchName)+2));
+        name = (char *)ckalloc((strlen(switchName)+2));
         *name = '-';
         strcpy(name+1, switchName);
     } else {
@@ -2463,7 +2442,7 @@ Itk_RemoveArchOptionPart(
      *  If the switch does not have a leading "-", add it on.
      */
     if (*switchName != '-') {
-        name = ckalloc((unsigned)(strlen(switchName)+2));
+        name = (char *)ckalloc(strlen(switchName)+2);
         *name = '-';
         strcpy(name+1, switchName);
     } else {
@@ -2600,8 +2579,8 @@ Itk_IgnoreArchOptionPart(
  * ------------------------------------------------------------------------
  */
 void
-Itk_DelOptionPart(optPart)
-    ArchOptionPart *optPart;  /* option part data to be destroyed */
+Itk_DelOptionPart(
+    ArchOptionPart *optPart)  /* option part data to be destroyed */
 {
     if (optPart->clientData && optPart->deleteProc) {
         (*optPart->deleteProc)(optPart->clientData);
@@ -2627,10 +2606,10 @@ Itk_DelOptionPart(optPart)
  * ------------------------------------------------------------------------
  */
 static ConfigCmdline*
-Itk_CreateConfigCmdline(interp, accessCmd, switchName)
-    Tcl_Interp *interp;              /* interpreter handling this request */
-    Tcl_Command accessCmd;           /* command for <object> being config'd */
-    char *switchName;                /* switch name of option being config'd */
+Itk_CreateConfigCmdline(
+    Tcl_Interp *interp,              /* interpreter handling this request */
+    Tcl_Command accessCmd,           /* command for <object> being config'd */
+    char *switchName)                /* switch name of option being config'd */
 {
     int i;
     ConfigCmdline *cmdlinePtr;
@@ -2664,8 +2643,8 @@ Itk_CreateConfigCmdline(interp, accessCmd, switchName)
  * ------------------------------------------------------------------------
  */
 static void
-Itk_DeleteConfigCmdline(cdata)
-    ClientData cdata;                /* command to be freed */
+Itk_DeleteConfigCmdline(
+    ClientData cdata)                /* command to be freed */
 {
     ConfigCmdline *cmdlinePtr = (ConfigCmdline*)cdata;
     int i;
@@ -2702,9 +2681,9 @@ Itk_DeleteConfigCmdline(cdata)
  * ------------------------------------------------------------------------
  */
 static Tcl_HashTable*
-Itk_CreateGenericOptTable(interp, options)
-    Tcl_Interp *interp;          /* interpreter handling this request */
-    const char *options;               /* string description of config options */
+Itk_CreateGenericOptTable(
+    Tcl_Interp *interp,          /* interpreter handling this request */
+    const char *options)               /* string description of config options */
 {
     int confc;
     const char **confv = NULL;
@@ -2774,8 +2753,8 @@ tableFail:
  * ------------------------------------------------------------------------
  */
 static void
-Itk_DelGenericOptTable(tPtr)
-    Tcl_HashTable *tPtr;  /* option table to be destroyed */
+Itk_DelGenericOptTable(
+    Tcl_HashTable *tPtr)  /* option table to be destroyed */
 {
     Tcl_HashEntry *entry;
     Tcl_HashSearch place;
@@ -2815,10 +2794,10 @@ Itk_DelGenericOptTable(tPtr)
  * ------------------------------------------------------------------------
  */
 static GenericConfigOpt*
-Itk_CreateGenericOpt(interp, switchName, accessCmd)
-    Tcl_Interp *interp;          /* interpreter handling this request */
-    const char *switchName;      /* command-line switch for option */
-    Tcl_Command accessCmd;       /* access command for component */
+Itk_CreateGenericOpt(
+    Tcl_Interp *interp,          /* interpreter handling this request */
+    const char *switchName,      /* command-line switch for option */
+    Tcl_Command accessCmd)       /* access command for component */
 {
     GenericConfigOpt *genericOpt = NULL;
     Tcl_Obj *codePtr = NULL;
@@ -2834,7 +2813,7 @@ Itk_CreateGenericOpt(interp, switchName, accessCmd)
      *  If the switch does not have a leading "-", add it on.
      */
     if (*switchName != '-') {
-        name = ckalloc((unsigned)(strlen(switchName)+2));
+        name = (char *)ckalloc((strlen(switchName)+2));
         *name = '-';
         strcpy(name+1, switchName);
 	my_name = name;
@@ -2889,7 +2868,7 @@ Itk_CreateGenericOpt(interp, switchName, accessCmd)
 
 optionDone:
     if (my_name != switchName) {
-        ckfree(my_name);
+        ckfree((char *)my_name);
     }
     if (codePtr) {
         Tcl_DecrRefCount(codePtr);
@@ -2911,8 +2890,8 @@ optionDone:
  * ------------------------------------------------------------------------
  */
 static void
-Itk_DelGenericOpt(opt)
-    GenericConfigOpt *opt;  /* option info to be destroyed */
+Itk_DelGenericOpt(
+    GenericConfigOpt *opt)  /* option info to be destroyed */
 {
     ckfree((char*)opt->storage);
     ckfree((char*)opt);
