@@ -26,12 +26,12 @@
 /*
  *  FORWARD DECLARATIONS
  */
-static char* ItkTraceClassDestroy _ANSI_ARGS_((ClientData cdata,
-    Tcl_Interp *interp, const char *name1, const char *name2, int flags));
-static Tcl_HashTable* ItkGetClassesWithOptInfo _ANSI_ARGS_((
-    Tcl_Interp *interp));
-static void ItkFreeClassesWithOptInfo _ANSI_ARGS_((ClientData cdata,
-    Tcl_Interp *interp));
+static char* ItkTraceClassDestroy (ClientData cdata,
+    Tcl_Interp *interp, const char *name1, const char *name2, int flags);
+static Tcl_HashTable* ItkGetClassesWithOptInfo (
+    Tcl_Interp *interp);
+static void ItkFreeClassesWithOptInfo (ClientData cdata,
+    Tcl_Interp *interp);
 
 
 /*
@@ -59,7 +59,7 @@ Itk_ClassOptionDefineCmd(
     ClientData clientData,   /* class parser info */
     Tcl_Interp *interp,      /* current interpreter */
     int objc,                /* number of arguments */
-    Tcl_Obj *CONST objv[])   /* argument objects */
+    Tcl_Obj *const objv[])   /* argument objects */
 {
     ItclObjectInfo *info = (ItclObjectInfo*)clientData;
     ItclClass *iclsPtr = (ItclClass*)Itcl_PeekStack(&info->clsStack);
@@ -180,7 +180,7 @@ Itk_ClassOptionIllegalCmd(
     ClientData clientData,   /* class parser info */
     Tcl_Interp *interp,      /* current interpreter */
     int objc,                /* number of arguments */
-    Tcl_Obj *CONST objv[])   /* argument objects */
+    Tcl_Obj *const objv[])   /* argument objects */
 {
     char *op = Tcl_GetString(objv[0]);
 
@@ -212,11 +212,12 @@ Itk_ConfigClassOption(
     Tcl_Interp *interp,        /* interpreter managing the class */
     ItclObject *contextObj,    /* object being configured */
     ClientData cdata,          /* class option */
-    CONST char *newval)        /* new value for this option */
+    const char *newval)        /* new value for this option */
 {
     ItkClassOption *opt = (ItkClassOption*)cdata;
     int result = TCL_OK;
     ItclMemberCode *mcode;
+    Tcl_CallFrame frame;
 
     /*
      *  If the option has any config code, execute it now.
@@ -224,17 +225,63 @@ Itk_ConfigClassOption(
      */
     mcode = opt->codePtr;
     if (mcode && mcode->bodyPtr) {
-//fprintf(stderr, "EXE!%s!\n", Tcl_GetString(mcode->bodyPtr));
-        Itcl_SetCallFrameResolver(interp, opt->iclsPtr->resolvePtr);
-        Tcl_Namespace *saveNsPtr = Tcl_GetCurrentNamespace(interp);
-//fprintf(stderr, "MCNS!%s!\n", saveNsPtr->fullName);
-        Itcl_SetCallFrameNamespace(interp, opt->iclsPtr->nsPtr);
+
+	Itcl_PushCallFrame(interp, &frame, opt->iclsPtr->nsPtr, 1);
+	Itcl_SetContext(interp, contextObj);
+
         result = Tcl_EvalObjEx(interp, mcode->bodyPtr, 0);
-        Itcl_SetCallFrameNamespace(interp, saveNsPtr);
-#ifdef NOTDEF
+
+	Itcl_UnsetContext(interp);
+	Itcl_PopCallFrame(interp);
+
+	/*
+	 * Here we engage in some ugly hackery workaround until
+	 * someone has time to come back and implement this
+	 * properly.
+	 *
+	 * In Itcl/Itk 3, the same machinery was used to implement
+	 * method invocation and configbody invocation, and the
+	 * code here looked like:
+	 *
+
         result = Itcl_EvalMemberCode(interp, (ItclMemberFunc*)NULL,
             opt->member, contextObj, 0, (Tcl_Obj**)NULL);
-#endif
+
+	 *
+	 * In Itcl 4, Itcl methods have become (a particular variant)
+	 * of TclOO methods.  It's not clear whether config bodies
+	 * should also do that, or what?
+	 *
+	 * Instead the existing solution above has been to just "eval"
+	 * the configbody body script in a suitable context, which
+	 * works very nearly correctly.  The trouble is that unlike
+	 * method invocation, we've not pushed a proper frame, nor
+	 * have we unwound a return level, so when the "eval" returns
+	 * TCL_RETURN we've not been handling that right.  You will
+	 * find some configbody bodies out there that expect to be
+	 * able to use [return] for early exit.  Iwidgets test
+	 * Extbutton-2.8 is an example.
+	 *
+	 * As a cheap workaround, we put in explicit special treatment
+	 * for (result == TCL_RETURN) here.  This is essentially a
+	 * reproduction of the Tcl internal routine TclUpdateReturnInfo()
+	 * but without the benefit of internals access.
+ 	 */
+
+	if (result == TCL_RETURN) {
+	    Tcl_Obj *opts = Tcl_GetReturnOptions(interp, TCL_RETURN);
+	    Tcl_Obj *levelKey = Tcl_NewStringObj("-level", -1);
+	    Tcl_Obj *levelObj;
+	    int level;
+
+	    Tcl_DictObjGet(NULL, opts, levelKey, &levelObj);
+	    Tcl_GetIntFromObj(NULL, levelObj, &level);
+
+	    Tcl_DictObjPut(NULL, opts, levelKey, Tcl_NewIntObj(--level));
+	    result = Tcl_SetReturnOptions(interp, opts);
+
+	    Tcl_DecrRefCount(levelKey);
+	}
     }
     return result;
 }
@@ -455,13 +502,13 @@ Itk_CreateClassOption(
     Tcl_IncrRefCount(opt->fullNamePtr);
     opt->codePtr = mcode;
 
-    opt->resName = (char*)ckalloc((unsigned)(strlen(resName)+1));
+    opt->resName = (char*)ckalloc((strlen(resName)+1));
     strcpy(opt->resName, resName);
 
-    opt->resClass = (char*)ckalloc((unsigned)(strlen(resClass)+1));
+    opt->resClass = (char*)ckalloc((strlen(resClass)+1));
     strcpy(opt->resClass, resClass);
 
-    opt->init = (char*)ckalloc((unsigned)(strlen(defVal)+1));
+    opt->init = (char*)ckalloc((strlen(defVal)+1));
     strcpy(opt->init, defVal);
 
     *optPtr = opt;
